@@ -1,6 +1,8 @@
 //! Server module
-use std::io;
+use std::fs::File;
+use std::io::{self, Read};
 use std::net::{IpAddr, SocketAddr as StdSocketAddr, ToSocketAddrs};
+use std::path::PathBuf;
 use std::pin::Pin;
 use std::task::{Context, Poll};
 
@@ -233,3 +235,27 @@ impl<I: Into<IpAddr>> IntoAddrIncoming for (I, u16) {
         incoming
     }
 }
+
+pub(crate) struct LazyFile {
+    path: PathBuf,
+    file: Option<File>,
+}
+
+impl LazyFile {
+    fn lazy_read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
+        if self.file.is_none() {
+            self.file = Some(File::open(&self.path)?);
+        }
+
+        self.file.as_mut().unwrap().read(buf)
+    }
+}
+impl Read for LazyFile {
+    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
+        self.lazy_read(buf).map_err(|err| {
+            let kind = err.kind();
+            tracing::error!(path = ?self.path, error = ?err, "error reading file");
+            io::Error::new(kind, format!("error reading file ({:?}): {}", self.path.display(), err))
+        })
+    }
+}
